<?php

class MoBackupSite{

function __construct()
{
	add_filter( 'cron_schedules', array($this,'db_eb_backup_interval'));
	add_action( 'mo_eb_bl_cron_hook', array($this,'db_cron_backup') );
	add_filter( 'cron_schedules', array($this,'file_eb_backup_interval'));
	add_action( 'mo_eb_file_cron_hook', array($this,'file_cron_backup') );
}
    
function db_cron_backup(){

		$obj = new MoBackupSite;
		$obj->backupDB();
	
}

function db_eb_backup_interval($schedules){
	$mo2f_cron_hours = get_site_option('mo_wpns_backup_time')*3600;
	$schedules['db_eb_backup_time'] = array(
		'interval' => $mo2f_cron_hours,
		'display'  => esc_html__( 'Cron Activated' ),
	);
    return $schedules;
}

function bl_deactivate() {
	$timestamp = wp_next_scheduled( 'mo_eb_bl_cron_hook' );
	wp_unschedule_event( $timestamp, 'mo_eb_bl_cron_hook' );
}

function file_cron_backup(){
  $homepath = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR;
  $backup_store_path = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR.'wp-content'.DIRECTORY_SEPARATOR.'uploads'.DIRECTORY_SEPARATOR;
  $time = time();
  update_site_option('backup_created_time',$time);

	if(get_site_option('mo_file_backup_plugins') =='1'){
		$this->plugin_backup($homepath, $backup_store_path, $time);
	}
	if(get_site_option('mo_file_backup_themes')=='1'){
      $this->themes_backup($homepath, $backup_store_path ,$time);  
	}

	if(get_site_option('mo_file_backup_wp_files') == '1'){
		$this->wpfiles_backup($homepath, $backup_store_path, $time);
	}
  		
}

function file_manual_backup(){
  $homepath = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR;
  $backup_store_path = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR.'wp-content'.DIRECTORY_SEPARATOR.'uploads'.DIRECTORY_SEPARATOR;
  $time = time();
  update_site_option('backup_created_time',$time);

	if(get_site_option('mo_file_manual_backup_plugins') =='1'){
		$this->plugin_backup($homepath, $backup_store_path, $time);
	}
	if(get_site_option('mo_file_manual_backup_themes')=='1'){
      $this->themes_backup($homepath, $backup_store_path ,$time);  
	}

	if(get_site_option('mo_file_manual_backup_wp_files') == '1'){
		$this->wpfiles_backup($homepath, $backup_store_path, $time);
	}
  		
}

function file_eb_backup_interval($schedules){
	$mo2f_cron_file_backup_hours = get_site_option('mo_wpns_backup_time')*3600;
	$schedules['file_eb_backup_time'] = array(
		'interval' => $mo2f_cron_file_backup_hours,
		'display'  => esc_html__( 'Cron Activated' ),
	);
 return $schedules;
}

function file_backup_deactivate(){
	$timestamp = wp_next_scheduled( 'mo_eb_file_cron_hook' );
	wp_unschedule_event( $timestamp, 'mo_eb_file_cron_hook' );
 }

 function plugin_backup($homepath, $backup_store_path, $time){
    global $wpnsDbQueries;
        $this->mkdirectory('plugins');
        $real_path= $homepath.'wp-content'.DIRECTORY_SEPARATOR.'plugins';
        $backup_path =$backup_store_path.'miniorangebackup'.DIRECTORY_SEPARATOR.'file-backups'.DIRECTORY_SEPARATOR.'plugins';
		$filename = 'miniorange-plugins-backup-'.$time.'.zip';
		$this->file_backup($real_path,$filename,'plugins');
	 $wpnsDbQueries->insert_backup_detail(MoWpnsConstants::PLUGIN,$filename,$time,$backup_path);
}

 function themes_backup($homepath, $backup_store_path ,$time){
    global $wpnsDbQueries;
        $this->mkdirectory('themes');
		$real_path= $homepath.'wp-content'.DIRECTORY_SEPARATOR.'themes';
		$backup_path =$backup_store_path.'miniorangebackup'.DIRECTORY_SEPARATOR.'file-backups'.DIRECTORY_SEPARATOR.'themes';
		$filename = 'miniorange-themes-backup-'.$time.'.zip';
		$this->file_backup($real_path,$filename,'themes');
	    $wpnsDbQueries->insert_backup_detail(MoWpnsConstants::THEMES,$filename,$time,$backup_path);

 }

 function wpfiles_backup($homepath, $backup_store_path, $time){
    global $wpnsDbQueries;
 	    $this->mkdirectory('wp_files');
		$real_path= $homepath;
		$backup_path =$backup_store_path.'miniorangebackup'.DIRECTORY_SEPARATOR.'file-backups'.DIRECTORY_SEPARATOR.'wp_files';
		$filename = 'miniorange-wpfiles-backup-'.$time.'.zip';
		$this->file_backup($real_path,$filename, 'wp_files');
	    $wpnsDbQueries->insert_backup_detail(MoWpnsConstants::WPFILES,$filename,$time,$backup_path);
 }

 function mkdirectory($foldername){
    
   $homepath = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR.'wp-content'.DIRECTORY_SEPARATOR.'uploads'.DIRECTORY_SEPARATOR;
   if(!is_writable($homepath)){
     wp_send_json('not_writable');
     return;
   }


   $basepath = $homepath;
   if(!file_exists($basepath."miniorangebackup")){
		mkdir($basepath."miniorangebackup");
    }

    $basepath = $homepath.'miniorangebackup'.DIRECTORY_SEPARATOR;
    $this-> create_index_file($basepath);
		      
    if(!file_exists($basepath.'file-backups')){
	     mkdir($basepath.'file-backups');
    }

    $basepath = $homepath.'miniorangebackup'.DIRECTORY_SEPARATOR.'file-backups'.DIRECTORY_SEPARATOR;
    if(!file_exists($basepath.$foldername)){
     	mkdir($basepath.$foldername);
    }
           
}

function create_index_file($folder_path){

	$html_path=$folder_path."index.html";
	$htaccess_path= $folder_path.".htaccess";

	if(!file_exists($html_path)){
        $f = fopen($html_path, "a");
        fwrite($f, '<html><body><a href="https://security.miniorange.com/" target="_blank">WordPress backups by miniorange</a></body></html>');
        fclose($f);
    }
    if(!file_exists($htaccess_path)){
    	$f = fopen($htaccess_path, "a");
    	fwrite($f, "deny from all");
    	fclose($f);
    }
}

function randomPassword() {
	$alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
	$pass = array(); //remember to declare $pass as an array
	$alphaLength = strlen($alphabet) - 1; //put the length -1 in cache
	for ($i = 0; $i < 16; $i++) {
		$n = rand(0, $alphaLength);
		$pass[] = $alphabet[$n];
	}
	return implode($pass); //turn the array into a string
 }

function send_email_backup($pass) {
	$toEmail = get_site_option('admin_email');
	$style="<style>.button {background-color: #008CBA;border: none;color: white;text-align: center;
	text-decoration: none;display: inline-block;font-size: 16px;padding: 14px 40px;margin: 4px 2px;cursor: pointer;}</style>";

	$content=$style.'<table cellpadding="25" style="margin:0px auto"><tbody><tr><td><table cellpadding="24" width="584px" style="margin:0 auto;max-width:584px;background-color:#f6f4f4;border:1px solid #a8adad">
	        <tbody><tr><td><img src="https://ci3.googleusercontent.com/proxy/bsqfwxlN_rHFOhApsbPGugF_GTN5hDO9LSLj6XI-u5TRUBW2scP-4M6HDfkRrGLKd5VLbNV_zI4V1jXKwsjOEvf0woDkXYbmbKhgnNYfbfdqari89aTVuY0mVQ=s0-d-e1-ft#https://miniorange.s3.amazonaws.com/public/images/miniorange-logo.png" style="color:#5fb336;text-decoration:none;display:block;width:auto;height:auto;max-height:35px"></td>
	        </tr></tbody></table><table cellpadding="24" style="background:#fff;border:1px solid #a8adad;width:584px;border-top:none;color:#4d4b48;font-family:Arial,Helvetica,sans-serif;font-size:13px;line-height:18px">
	        <tbody><tr><td>
	         <p style="margin-top:0;margin-bottom:20px">Dear User,</p><p style="margin-top:0;margin-bottom:10px"><p style="margin-top:0;margin-bottom:10px">A backup for your all folder has been created for you. The ZIP is password protected and password is <b>'.$pass.'</b> for all backup folder and further the password will be same  for all backup until you receive the mail for new password. </p></p>
	        <p style="margin-top:0;margin-bottom:10px"><p style="margin-top:0;margin-bottom:10px">Your backup is created under "/WordPress/file-backups" directory<br><br><p style="margin-top:0;margin-bottom:15px">Thank you,<br>miniOrange Team</p><p style="margin-top:0;margin-bottom:0px;font-size:11px">Disclaimer: This email and any files transmitted with it are confidential and intended solely for the use of the individual or entity to whom they are addressed.</p>
	        </span></td></tr></tbody></table></td></tr></tbody></table>';
	$headers = array('Content-Type: text/html; charset=UTF-8');
	$subject='Backup For Database';
    $sent= wp_mail( $toEmail, $subject, $content, $headers);
}

function file_backup($real_path, $filename, $foldername){
	ini_set('max_execution_time', 0);
     $backup_store_path = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR.'wp-content'.DIRECTORY_SEPARATOR.'uploads'.DIRECTORY_SEPARATOR.'miniorangebackup'.DIRECTORY_SEPARATOR.'file-backups'.DIRECTORY_SEPARATOR;
    $rootPath = realpath($real_path);
    $zip = new ZipArchive();
    $res = $zip->open($backup_store_path.$foldername.DIRECTORY_SEPARATOR.$filename, ZipArchive::CREATE | ZipArchive::OVERWRITE);

    $files = new RecursiveIteratorIterator(
	    new RecursiveDirectoryIterator($rootPath),
	    RecursiveIteratorIterator::LEAVES_ONLY
	);
	foreach ($files as $name => $file)
	{
	   if (!$file->isDir())
	    {
	       $filePath = $file->getRealPath();
	       $relativePath = substr($filePath, strlen($rootPath) + 1);
           if(strpos($relativePath, 'miniorangebackup')!== false ){}
           else{
               $zip->addFile($filePath, $relativePath);
            }
        }
    }   
	$zip->close();
}


function backupDB(){
		
	if ( function_exists('memory_get_usage') && ( (int) ini_get('memory_limit') < 128 ) ){
		ini_set('memory_limit', '128M' );
		do_action('mo_eb_show_message',MoBackupMessages::showMessage('DB_MEMORY_LIMIT'),'SUCCESS');
	}

    $backup_store_path = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR.'wp-content'.DIRECTORY_SEPARATOR.'uploads'.DIRECTORY_SEPARATOR;

    if(!is_writable($backup_store_path)){
    	wp_send_json('not_writable');
    	return;
    }
	global $wpdb;
	$tables 		= $wpdb->get_results("SHOW TABLES", ARRAY_N);
	$nooftables 	= count($tables);
	$query			= "";
	$tableswithfk 	= array();
	$tableswithoutfk= array();

	foreach($tables as $table)
	{
		if(is_array($table))
			$table = $table[0];
		$createtable = $wpdb->get_results("SHOW CREATE TABLE  $table", ARRAY_A);
		if(!empty($createtable[0]))
		{
			$createquery = $createtable[0]['Create Table'];
			if (strpos($createquery, 'FOREIGN KEY') !== false) 
				array_push($tableswithfk,$table);
			else
				array_push($tableswithoutfk, $table);
		}
	}
	
	$query .= $this->get_table_query($query,$tableswithoutfk);

	$query .= $this->get_table_query($query,$tableswithfk);

	$fileName = $this->create_db_backup_file($query);
	wp_send_json('created backup');
}

function get_table_query($query,$tables)
{

global $wpdb;
foreach($tables as $table)
{
	$createtable = $wpdb->get_results("SHOW CREATE TABLE  $table", ARRAY_A);
	if(!empty($createtable[0]))
	{		
		$createquery = $createtable[0]['Create Table'];		
		$query 	    .= 'DROP TABLE IF EXISTS '.$table.";\n";
		$query 	    .= $createquery.";\n\n";
		$data    	 = $wpdb->get_results("SELECT * FROM $table", ARRAY_A);
		foreach($data as $record)
		{
			if(count($record)>0)
			{
				$query.= 'INSERT INTO '.$table.' VALUES(';
				$i=0;
				foreach($record as $key=>$value)
				{
					$value = addslashes($value);
					if (isset($value))
						$query.= '"'.$value.'"' ;
					else
						$query.= '""';
					if ($i < (count($record)-1)) { $query.= ','; }
					$i++;
				}
				$query.= ");\n";
			}
		}
		$query.="\n\n";
	}
}
return $query;
}

function create_db_backup_file($data)
{  

	global $wpnsDbQueries;
    $time = time();
	$basepath  = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR;
	$backup_store_path = dirname(dirname(dirname(dirname(dirname(__FILE__))))).DIRECTORY_SEPARATOR.'wp-content'.DIRECTORY_SEPARATOR.'uploads'.DIRECTORY_SEPARATOR;
	if(!file_exists($backup_store_path."miniorangebackup")){
			mkdir($backup_store_path."miniorangebackup");
		}
        $basepath = $backup_store_path.'miniorangebackup'.DIRECTORY_SEPARATOR;
		$handler_obj = new MoBackupSite;
		$handler_obj->create_index_file($basepath);
		if(!file_exists($basepath.'db-backups')){
			mkdir($basepath.'db-backups');
		}

    $backup_path = $basepath.'db-backups';	 
	$filename = 'miniorange-db-backup-'.$time.'.sql';
	$basepath = $basepath.'db-backups';
	$handle = fopen($basepath.DIRECTORY_SEPARATOR.$filename,'w+');
	fwrite($handle,$data);
	fclose($handle);
	$wpnsDbQueries->insert_backup_detail(MoWpnsConstants::DATABASE,$filename,$time,$backup_path);
	return $filename;
}
}new MoBackupSite;