<?php
class Mo_wpns_scan_malware
{
	function __construct(){
		add_action( 'admin_init'  , array( $this, 'mo_wpns_malware_scanner_functions' ) );
	}

	public function mo_wpns_malware_scanner_functions(){
		add_action('wp_ajax_mo_wpns_malware_redirect', array( $this, 'mo_wpns_malware_redirect' ));
	}

	public function mo_wpns_malware_redirect(){
		switch($_POST['call_type'])
		{
			case "submit_malware_settings_form":
				$this->mo_wpns_save_malware_config_form($_POST);
				break;
			case "malware_scan_initiate":
				$this->mo_wpns_start_malware_scan($_POST);
				break;
			case "wpns_enable_tour":
				update_option('skip_tour', 0);
				break;
			case "wpns_disable_tour":
				update_option('skip_tour', 1);
				break;
			case "malware_progress_bar":
				$this->mo_wpns_get_progress();
				break;
			case "malware_scan_last_result":
				$this->mo_wpns_get_scan_result();
				break;
			case "malware_scan_terminate":
				$this->mo_wpns_stop_scan();
				break;
		}
	}

	public function mo_wpns_save_malware_config_form($POSTED){
		$nonce = $POSTED['nonce'];
	   		if ( ! wp_verify_nonce( $nonce, 'mo-wpns-scan-settings' ) ){
	   			wp_send_json('ERROR');
	   			return;
	   		}
		if(! isset($POSTED['scan_plugin']) && ! isset($POSTED['scan_themes']) && ! isset($POSTED['scan_core'])){
			wp_send_json('folder_error');
		}elseif (! isset($POSTED['vulnerable_check']) && ! isset($POSTED['sql_check']) && ! isset($POSTED['rfi_check']) && ! isset($POSTED['ext_link']) && ! isset($POSTED['repo_check'])) {
			wp_send_json('level_error');
		}
		else{
			isset($POSTED['scan_plugin']) ? update_option('mo_wpns_scan_plugins',sanitize_text_field($POSTED['scan_plugin'])) : update_option('mo_wpns_scan_plugins',0);
			isset($POSTED['scan_themes']) ? update_option('mo_wpns_scan_themes',sanitize_text_field($POSTED['scan_themes'])) : update_option('mo_wpns_scan_themes',0);
			isset($POSTED['scan_core']) ? update_option('mo_wpns_scan_wp_files',sanitize_text_field($POSTED['scan_core'])) : update_option('mo_wpns_scan_wp_files',0);
			update_option('mo_wpns_scan_files_extensions',sanitize_text_field($POSTED['file_type']));
			isset($POSTED['vulnerable_check']) ? update_option('mo_wpns_check_vulnerable_code',sanitize_text_field($POSTED['vulnerable_check'])) : update_option('mo_wpns_check_vulnerable_code',0);
			isset($POSTED['sql_check']) ? update_option('mo_wpns_check_sql_injection',sanitize_text_field($POSTED['sql_check'])) : update_option('mo_wpns_check_sql_injection',0);
			isset($POSTED['ext_link']) ? update_option('mo_wpns_check_external_link',sanitize_text_field($POSTED['ext_link'])) : update_option('mo_wpns_check_external_link',0);
			isset($POSTED['repo_check']) ? update_option('mo_wpns_scan_files_with_repo',sanitize_text_field($POSTED['repo_check'])) : update_option('mo_wpns_scan_files_with_repo',0);
			$folders_to_skip= $POSTED['skip_path'];
			$folders_to_skip = str_replace('\\\\', '/', $folders_to_skip);
			$str="";
			if(!empty($folders_to_skip)){
				$folders_to_skip_array=explode(";",$folders_to_skip);
				for($i=0; $i< count($folders_to_skip_array); $i++){
					if(is_dir($folders_to_skip_array[$i])){
						$str.= $folders_to_skip_array[$i];
						if($i!= count($folders_to_skip_array)-1){
							$str.=";";
						}
					}elseif($folders_to_skip_array[$i] == ''){
						$str.= $folders_to_skip_array[$i];
					}
					else{
						wp_send_json('path_error');
					}
				}
				update_option( 'mo_wpns_skip_folders', $str);
			}else{
				update_option('mo_wpns_skip_folders', $folders_to_skip);
			}
			wp_send_json('save_success');
		}
	}

	public function mo_wpns_start_malware_scan($POSTED){
		if(get_option('mo_wpns_malware_scan_in_progress') == "IN PROGRESS"){
			wp_send_json('scanning_already');
		}
        global $moWpnsUtility, $mo2f_dirName;

		$mo_wpns_scan_handler = new Mo_wpns_Scan_Handler_Cron();
		$mo2f_malware_db_handler = new MoWpnsDB();

		$current_time= time();
		update_option('mo_wpns_last_scan_time', $current_time);
		$plugin_current= get_plugins();
		update_option('mo_wpns_last_plugins', $plugin_current);
		$args=array();
		$theme_current= wp_get_themes($args);
		update_option('mo_wpns_last_themes', $theme_current);

		update_option('mo2f_scan_initialize', 0);
		update_option('mo_wpns_malware_scan_in_progress','IN PROGRESS');
		update_option('mo_wpns_files_scanned',0);
		update_option('mo_wpns_infected_files',0);
		update_option('wordpress_download_status', false);
		$mo2f_malware_db_handler->delete_files_parts();

		if(isset($POSTED['scan']) && $POSTED['scan']=='scan_start'){
			$scan_configuration = array();
			switch($POSTED['scantype'])
			{
				case "quick_scan":
					$nonce = $POSTED['nonce'];
			   		if ( ! wp_verify_nonce( $nonce, 'wpns-quick-scan' ) ){
			   			wp_send_json('ERROR');
			   			return;
			   		}
					$scan_configuration = MoWpnsConstants::$quick_scan_configuration;
					update_option('mo_wpns_scan_mode','quick_scan');
					$mo_wpns_scan_handler->mo2f_scan_all_files($scan_configuration, $current_time); //recheck
					break;
				case "standard_scan":
					$nonce = $POSTED['nonce'];
			   		if ( ! wp_verify_nonce( $nonce, 'wpns-standard-scan' ) ){
			   			wp_send_json('ERROR');
			   			return;
			   		}
					$scan_configuration = MoWpnsConstants::$standard_scan_configuration;
					update_option('mo_wpns_scan_mode','standard_scan');
					$mo_wpns_scan_handler->mo2f_scan_all_files($scan_configuration, $current_time); //recheck
					break;
				case "custom_scan":
					$nonce = $POSTED['nonce'];
			   		if ( ! wp_verify_nonce( $nonce, 'wpns-custom-scan' ) ){
			   			wp_send_json('ERROR');
			   			return;
			   		}
					$scan_configuration = array('plugin_scan' => get_option('mo_wpns_scan_plugins') ,
												'theme_scan'  => get_option('mo_wpns_scan_themes'),
												'core_scan'	  => get_option('mo_wpns_scan_wp_files'),
												'file_extension' => get_option('mo_wpns_scan_files_extensions'),
												'check_vulnerable' => get_option('mo_wpns_check_vulnerable_code') ,
												'check_sql' => get_option('mo_wpns_check_sql_injection'),
												'check_rfi' => get_option('mo_wpns_check_remote_file_inclusion'),
												'ext_link_check' => get_option('mo_wpns_check_external_link'),
												'check_repo' => get_option('mo_wpns_scan_files_with_repo'),
												'path_skip' => get_option('mo_wpns_skip_folders'),
												'type_scan' => "Custom Scan"
												);
					update_option('mo_wpns_scan_mode','custom_scan');
					update_option('mo2f_custom_scan_config', $scan_configuration);
					$mo_wpns_scan_handler->mo2f_scan_all_files($scan_configuration, $current_time); //recheck
					break;
			}
		}
	}


	public function mo_wpns_get_progress(){
		$status= get_option('mo_wpns_malware_scan_in_progress');
		$files_scanned= get_option('mo_wpns_files_scanned');
		$total_files = get_option('mo_wpns_total_files');
		$check_repo = get_option('mo_wpns_scan_files_with_repo');
		$scan_mode= get_option('mo_wpns_scan_mode');
		$status_download= get_option('wordpress_download_status');
		if($status_download != false && $status == "IN PROGRESS"){
			$time_spent_in_downloading= time()-$status_download['time'];
			if($time_spent_in_downloading > 120){
				error_log("progress check");
				$mo2f_malware_db_handler = new MoWpnsDB();
				$reportid= get_option('mo2f_report_id');
				$last_report = $mo2f_malware_db_handler->get_report_with_id($reportid);
				if(!empty($last_report)){
					$response['file_count']= $last_report[0]->scanned_files;
					$response['malware_count'] = $last_report[0]->malware_count;
					$response['repo_issues'] = -96;
					$response['malicious_link'] = $last_report[0]->malicious_links;

					$mo2f_malware_db_handler->scan_report_complete($reportid, $response['file_count'], $response['malware_count'], $response['repo_issues'], $response['malicious_link']);
				}

				$scan_handler_obj = new mo2f_scanner_parts;
				$scan_handler_obj->mo2f_end_scan();
			}
		}
		$repo_scan=0;
		if($scan_mode == "standard_scan" || ($scan_mode == "custom_scan" && $check_repo == 1)){
			$repo_scan=1;
		}
		if($status == "IN PROGRESS"){
			$result=array("status"=>$status, "scanned"=>$files_scanned, "total"=>$total_files, "repo_scan"=> $repo_scan);
			wp_send_json($result);
		}
		else{
			//stop scan check
			$stop_scan=get_option('mo_stop_scan');

			$mo_wpns_db_handler = new MoWpnsDB();
			$total_scan=$mo_wpns_db_handler->count_files();
			$total_malicious=$mo_wpns_db_handler->count_malicious_files();
			$last_id=$mo_wpns_db_handler->get_last_id();
			$send_id=$last_id[0]->max;
			$last_scan=$mo_wpns_db_handler->count_files_last_scan($send_id);
			$malicious_last_scan= get_option('mo_wpns_infected_files');
			if($total_scan > 999){
				$total_scan=($total_scan/1000);
				$total_scan= round($total_scan,1)."k";
			}
			if($total_malicious > 999){
				$total_malicious=($total_malicious/1000);
				$total_malicious= round($total_malicious,1)."k";
			}
			if($stop_scan){
				$status="ABORTED";
				$result = array("status"=>$status, 'total_files'=>$total_scan, 'total_mal'=>$total_malicious, 'scan_files'=>$last_scan, 'mal_files'=>$malicious_last_scan);
				wp_send_json($result);
			}
			$result = array("status"=>$status, 'total_files'=>$total_scan, 'total_mal'=>$total_malicious, 'scan_files'=>$total_files, 'mal_files'=>$malicious_last_scan);
			wp_send_json($result);
		}
	}

	function mo_wpns_stop_scan(){
		error_log("Stop  Scan");
		update_option('mo_stop_scan','1');
		$mo2f_malware_db_handler = new MoWpnsDB();
		$mo2f_malware_db_handler->delete_files_parts();
		//update_option('mo_wpns_malware_scan_in_progress','COMPLETE');
		wp_send_json('success');
	}

	function mo_wpns_get_scan_result(){
		$status= get_option('mo_wpns_malware_scan_in_progress');
		$mo_wpns_db_handler = new MoWpnsDB();
		$result = $mo_wpns_db_handler->get_report();
		$str1= $_SERVER['REQUEST_URI'];
		$str1= str_replace("admin-ajax.php", "admin.php?page=mo_2fa_malwarescan", $str1);
		$currenturl = remove_query_arg('delete',$str1);
		$currenturl = remove_query_arg('view',$currenturl);
		$currenturl = remove_query_arg('trust',$currenturl);
		$currenturl = remove_query_arg('trustchanged',$currenturl);
		$htmldata = "";
		if(!empty($result)){
			foreach($result as $report){
				if(empty($report->scan_mode)){
					$report->scan_mode="Custom Scan";
				}
				$repo_count = $report->repo_issues >= 0 ? $report->repo_issues : 0;
				$htmldata .= "<tr><td style=text-align:center>".$report->scan_mode."</td>";
				$htmldata .= "<td style=text-align:center>";
				if(!empty($report->scanned_folders)){
					foreach(explode(";",$report->scanned_folders) as $folder){
						if(!empty($folder)){
							$htmldata .= $folder."<br>";
						}
					}
				}
				$htmldata .= "</td><td style=text-align:center>";
				$htmldata .= "<span style=color:green id=scan_files>".$report->scanned_files." files scanned<br></span>";
				$htmldata .= "<span style=color:red id=malicious_files>".$report->malware_count." malware found<br></span>";
				if($report->repo_issues < 0){
					$htmldata .= "<span style=color:red id=malicious_files>Issues with repository check<br></span>";
				}
				$htmldata .= "<span style=color:orange id=warning_files>".($repo_count+$report->malicious_links)." warnings found</span>";
				$htmldata .= "</td><td style=text-align:center id=start_time>".date("M j, Y, g:i:s a",$report->start_timestamp)."</td>";
				$htmldata .= "<td><a href='".add_query_arg( array('tab' => 'default', 'view' => $report->id), $currenturl )."'>View Details</a> <a href='".add_query_arg( array('tab' => 'default', 'delete' => $report->id), $currenturl )."'>Delete</a></td>";
				$htmldata .= "</tr>";
			
			}
		}else{
			$htmldata .= '<tr class="odd"><td valign="top" colspan="5" class="dataTables_empty">No data available in table</td></tr>';
		}
		wp_send_json($htmldata);
		
	}
}
new Mo_wpns_scan_malware();
?>